#include "kosSyst.h"
#include "KosFile.h"
#include "gfxdef.h"
#include "crc32.h"

extern "C" void __stdcall lzma_decompress(
	 const void* source,
	 void* destination,
	 unsigned dest_length);

//////  CKosBitmap

CKosBitmap::CKosBitmap()
{
	this->bmpID = -1;
	this->buffer = NULL;
	this->sizeX = 0;
	this->sizeY = 0;
}


CKosBitmap::~CKosBitmap()
{
	if ( this->buffer != NULL ) delete this->buffer;
}


//    
bool CKosBitmap::LoadFromArch( SCompBmpHeader *bmpArchDesc, CKosFile *fromFile, int ID )
{
	Byte *tmpBuff;
	
	//
	if ( this->buffer != NULL )
	{
		delete this->buffer;
		this->buffer = NULL;
	}
	//
	this->buffer = new RGB[bmpArchDesc->sizeX * bmpArchDesc->sizeY];
	//
	tmpBuff = new Byte[bmpArchDesc->compressedSize];
	//
	fromFile->Seek( SEEK_SET, bmpArchDesc->physicalOffset );
	if ( fromFile->Read( tmpBuff, bmpArchDesc->compressedSize ) == bmpArchDesc->compressedSize )
	{
		//
		if ( bmpArchDesc->compressedSize != bmpArchDesc->uncompressedSize )
		{
			//
			lzma_decompress( tmpBuff, this->buffer, bmpArchDesc->uncompressedSize);
		}
		else
		{
			//
			memcpy( (Byte *)(this->buffer), tmpBuff, bmpArchDesc->compressedSize );
		}
		//
		this->sizeX = bmpArchDesc->sizeX;
		this->sizeY = bmpArchDesc->sizeY;
		this->bmpID = ID;
	}
	//
	delete tmpBuff;
	//
	return true;
}


//    
void CKosBitmap::Draw( Word x, Word y )
{
	//
	if ( this->buffer != NULL )
		//
		kos_PutImage( this->buffer, this->sizeX, this->sizeY, x, y );
}


//     
RGB * CKosBitmap::GetBits()
{
	return this->buffer;
}


//   
void CKosBitmap::GetSize( Word &cx, Word &cy )
{
	cx = this->sizeX;
	cy = this->sizeY;
}



////////////////////// CFishka ///////////////////////

CFishka::CFishka( CKosBitmap *fromBmp, int yOffset, RGB insColour )
{
	int i, c;

	//
	this->bits = fromBmp->GetBits() + (yOffset * blockSize);
	this->transColour = insColour;
	//
	this->highLighted = new RGB[blockSize * blockSize];
	//
	for ( i = 0; i < (blockSize * blockSize); i++ )
	{
		//
		this->highLighted[i] = this->bits[i];
		//
		if ( this->highLighted[i] != this->transColour )
		{
			c = ( this->highLighted[i].b * 185 ) / 100;
			this->highLighted[i].b = (c > 255) ? 255 : c;
			c = ( this->highLighted[i].g * 185 ) / 100;
			this->highLighted[i].g = (c > 255) ? 255 : c;
			c = ( this->highLighted[i].r * 185 ) / 100;
			this->highLighted[i].r = (c > 255) ? 255 : c;
		}
	}
}

//
CFishka::~CFishka()
{
	//
	delete this->highLighted;
}


//
RGB * CFishka::GetBits()
{
	return this->bits;
}

//
RGB * CFishka::GetHighlightedBits()
{
	return this->highLighted;
}



